"""
This module contains SystemObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
SystemObjects.

    >>> o = r.object_add("/SystemObjects/Test")
    >>> o.add("000102030405")
    >>> print o.items
    ['000102030405']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil
import socket

class SystemObject(plobject.PLObject):
	"""
	SystemObject Implementation

	Object path: /SystemObjects

	@cvar Item: SystemObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	
	base_path = plutil.roConst("/SystemObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_SYSTEMOBJECT, "Number representing the type of object")

	def _create_item(self, value):
		if not isinstance(value, basestring):
			raise ValueError("String expected")

		item = self.Item(self, value=value)

		return item

	def add(self, value):
		"""
		Adds a SystemObject item to this SystemObject.

		@type value: C{str}
		@param value: The value is a string describing the SystemObject item.
			The string can be the IP protocol number or a string found in
			your services file. (GRE, TCP etc...)

		"""
		plobject.PLObject.add(self, value)


	class Item(plobject.PLObject.Item):
		"""
		SystemObject Item implementation.
		Should not be instantiated, use L{SystemObject.add} instead.

		@type protocol: C{int}
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(value=str, *plobject._item_props)

		def __str__(self):
			return str(self._prop.value)

		value = property(lambda self: self._prop.value, doc="SystemID.")
