"""
This module contains ContentlogicObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
ContentlogicObjects.

    >>> o = r.object_add("/ContentlogicObjects/Test")
    >>> o.add("Entertainment")
    >>> print o.items
    ['Entertainment']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil


class ContentlogicObject(plobject.PLObject):
    """
    ContentlogicObject Implementation, for categories

    Object path: /ContentlogicObjects

    @cvar Item: ContentlogicObject Item implementation.
    """

    __slots__ = ('_prop', '_pldb', '_callback', '_item_added')
    _may_exclude = True

    base_path = plutil.roConst("/ContentlogicObjects", "Only the type portion of the path")
    type = plutil.roConst(PLDB.OBJECT_TYPE_CONTENTLOGICOBJECT, "Number representing the type of object")

    def _create_item(self, value):
        return self.Item(self, value=value)

    def add(self, value, exclude=False):
        """
        Add a ContentlogicObject Item to this ContentlogicObject

        @type value: C{str}
        @param value: tag

        @type exclude: C{boolean}
        @param exclude: If this is set to True the tag in this item must be excluded,
                        otherwise the tag in this item must be included.

        """
        plobject.PLObject.add(self, value, exclude)

    class Item(plobject.PLObject.Item):
        """
        ContentlogicObject Item implementation.
        Should not be instantiated, use L{ContentlogicObject.add} instead.

        @type value: C{str}
        """
        __slots__ = ('_prop', '_parent')
        _Prop = plutil.Prop(exclude=bool, value=str, *plobject._item_props)

        def __str__(self):
            return "%s%s" % (['', '!'][self._prop.exclude], self._prop.value)

        value = property(lambda self: self._prop.value, doc="Contentlogic")
        exclude = property(lambda self: self._prop.exclude, doc="True = exclude, False = include")
